package org.jboss.chap2.ex3;

import java.lang.reflect.Method;
import javax.ejb.CreateException;
import javax.management.Attribute;
import javax.management.AttributeList;
import javax.management.AttributeNotFoundException;
import javax.management.DynamicMBean;
import javax.management.InvalidAttributeValueException;
import javax.management.JMRuntimeException;
import javax.management.MBeanAttributeInfo;
import javax.management.MBeanConstructorInfo;
import javax.management.MBeanInfo;
import javax.management.MBeanNotificationInfo;
import javax.management.MBeanOperationInfo;
import javax.management.MBeanException;
import javax.management.MBeanServer;
import javax.management.ObjectName;
import javax.management.ReflectionException;
import javax.naming.InitialContext;
import javax.naming.NamingException;

import org.jboss.system.ServiceMBeanSupport;

/**
 * Przykad komponentu typu DynamicMBean udostpniajcego wybrane atrybuty i
 * operacje komponentu EJB w postaci usugi MBean.
 * 
 * @author Scott.Stark@jboss.org
 * @version $Revision: 1.5 $
 */
public class EjbMBeanAdaptor extends ServiceMBeanSupport implements
		DynamicMBean {
	private String helloPrefix;

	private String ejbJndiName;

	private EchoLocalHome home;

	/**
	 * Atrybuty, ktre zostan udostpnione
	 */
	private MBeanAttributeInfo[] attributes = {
			new MBeanAttributeInfo("HelloPrefix", "java.lang.String",
					"Prefiks dodawany do wartoci zwracanej przez echo", true, // odczyt
					// (isReadable)
					true, // zapis (isWritable)
					false), // metoda is (isIs)
			new MBeanAttributeInfo("EjbJndiName", "java.lang.String",
					"Nazwa JNDI lokalnego interfejsu domowego", true, // odczyt
					true, // zapis
					false) // metoda is
	};

	/**
	 * Operacje, ktre zostan udostpnione
	 */
	private MBeanOperationInfo[] operations;

	/**
	 * Pokrywamy t metod w celu skonfigurowania operacji echo. Moemy tego
	 * rwnie dokona w ctor.
	 */
	public ObjectName preRegister(MBeanServer server, ObjectName name)
			throws Exception {
		log.info("komunikat preRegister");

		operations = new MBeanOperationInfo[5];

		Class thisClass = getClass();
		Class[] parameterTypes = { String.class };
		Method echoMethod = thisClass.getMethod("echo", parameterTypes);
		String desc = "Operacja echo wywouje metod echo komponentu sesyjnego i zwraca "
				+ "przekazan jej warto rozszerzon o prefix helloPrexif";
		operations[0] = new MBeanOperationInfo(desc, echoMethod);

		// Dodajemy zdefiniowane w nadklasie operacje interfejsu Service
		parameterTypes = new Class[0];
		Method createMethod = thisClass.getMethod("create", parameterTypes);
		operations[1] = new MBeanOperationInfo("JBoss Service.create",
				createMethod);
		Method startMethod = thisClass.getMethod("start", parameterTypes);
		operations[2] = new MBeanOperationInfo("JBoss Service.start",
				startMethod);
		Method stopMethod = thisClass.getMethod("stop", parameterTypes);
		operations[3] = new MBeanOperationInfo("JBoss Service.stop",
				startMethod);
		Method destroyMethod = thisClass.getMethod("destroy", parameterTypes);
		operations[4] = new MBeanOperationInfo("JBoss Service.destroy",
				startMethod);

		return name;
	}

	// --- Pocztek pokrycia metod interfejsu ServiceMBeanSupport
	protected void createService() throws Exception {
		log.info("stan create");
	}

	protected void startService() throws Exception {
		log.info("stan start");
		InitialContext ctx = new InitialContext();
		home = (EchoLocalHome) ctx.lookup(ejbJndiName);
		log.info("testowe Echo");
		EchoLocal bean = home.create();
		String echo = bean.echo("startService");
		log.info("echo(startService) = " + echo);
	}

	protected void stopService() {
		log.info("stan stop");
	}

	// --- Koniec pokrycia metod interfejsu ServiceMBeanSupport

	public String getHelloPrefix() {
		return helloPrefix;
	}

	public void setHelloPrefix(String helloPrefix) {
		this.helloPrefix = helloPrefix;
	}

	public String getEjbJndiName() {
		return ejbJndiName;
	}

	public void setEjbJndiName(String ejbJndiName) {
		this.ejbJndiName = ejbJndiName;
	}

	public String echo(String arg) throws CreateException, NamingException {
		log.info("Wyszukiwanie EchoLocalHome@" + ejbJndiName);
		EchoLocal bean = home.create();
		String echo = helloPrefix + bean.echo(arg);
		return echo;
	}

	// --- Pocztek pokrycia metod interfejsu DynamicMBean
	/**
	 * Zwraca interfejs zarzdzajcy opisujcy zasb dynamiczny. Nasza
	 * implementacja odpowiada za to, e opis ten jest dokadny i poprawny.
	 * 
	 * @return deskryptor interfejsu zarzdzajcego.
	 */
	public MBeanInfo getMBeanInfo() {
		String classname = getClass().getName();
		String description = "Komponent MBean uywajcy obiektu sesyjnego "
				+ "implementujcego operacj echo";
		MBeanConstructorInfo[] constructors = null;
		MBeanNotificationInfo[] notifications = null;
		MBeanInfo mbeanInfo = new MBeanInfo(classname, description, attributes,
				constructors, operations, notifications);
		// Zapisujemy do dziennika, znamy dziki temu moment wywoania tej
		// operacji
		Throwable trace = new Throwable("getMBeanInfo trace");
		log.info("Bez paniki, to tylko stos wywoania, nie wyjtek", trace);
		return mbeanInfo;
	}

	/**
	 * Zwraca warto atrybutu o nazwie odpowiadajcej przekazanemu acuchowi
	 * znakw
	 * 
	 * @param attribute
	 *            nazwa atrybutu.
	 * @return warto atrybutu.
	 * @exception AttributeNotFoundException
	 *                jeli nie ma atrybutu o danej nazwie.
	 * @exception MBeanException
	 *                owija bd zgaszany przez zasb podczas odwoania do
	 *                atrybutu.
	 * @exception ReflectionException
	 *                owija bd zgoszony podczas prby odwoania si do
	 *                atrybutu za pomoc refleksji.
	 */
	public Object getAttribute(String attribute)
			throws AttributeNotFoundException, MBeanException,
			ReflectionException {
		Object value = null;
		if (attribute.equals("HelloPrefix"))
			value = getHelloPrefix();
		else if (attribute.equals("EjbJndiName"))
			value = getEjbJndiName();
		else
			throw new AttributeNotFoundException(
					"Odwoanie do nieznanego atrybutu(" + attribute + ")");
		return value;
	}

	/**
	 * Zwraca wartoci atrybutw o nazwach bdcych elementami przekazywanej
	 * tablicy acuchw znakw
	 * 
	 * @param attributes
	 *            nazwy atrybutw.
	 * @return lista par nazwa/warto {@link AttributeList AttributeList}.
	 */
	public AttributeList getAttributes(String[] attributes) {
		AttributeList values = new AttributeList();
		for (int a = 0; a < attributes.length; a++) {
			String name = attributes[a];
			try {
				Object value = getAttribute(name);
				Attribute attr = new Attribute(name, value);
				values.add(attr);
			} catch (Exception e) {
				log.error("Nie znaleziono atrybutu: " + name, e);
			}
		}
		return values;
	}

	/**
	 * Ustala warto atrybutu. Atrybut i jego warto s przekazywane w postaci
	 * pary nazwa/warto {@link Attribute Attribute}
	 * 
	 * @see javax.management.Attribute
	 * 
	 * @param attribute
	 *            nazwa i nowa warto atrybutu.
	 * @exception AttributeNotFoundException
	 *                jeli dany atrybut nie zostanie znaleziony.
	 * @exception InvalidAttributeValueException
	 *                jeli nowa warto nie moe zosta dopasowana do typu
	 *                atrybutu
	 * @exception MBeanException
	 *                owija bd zgaszany przez zasb podczas odwoania do
	 *                atrybutu
	 * @exception ReflectionException
	 *                owija bd zgoszony podczas prby odwoania si do
	 *                atrybutu za pomoc refleksji
	 */
	public void setAttribute(Attribute attribute)
			throws AttributeNotFoundException, InvalidAttributeValueException,
			MBeanException, ReflectionException {
		String name = attribute.getName();
		if (name.equals("HelloPrefix")) {
			String value = attribute.getValue().toString();
			setHelloPrefix(value);
		} else if (name.equals("EjbJndiName")) {
			String value = attribute.getValue().toString();
			setEjbJndiName(value);
		} else
			throw new AttributeNotFoundException(
					"Odwoanie do nieznanego atrybutu(" + name + ")");
	}

	/**
	 * Ustala wartoci atrybutw na podstawie przekazanej listy, ktra zawiera
	 * elementy bdce par nazwa/warto {@link AttributeList AttributeList}
	 * 
	 * @param attributes
	 *            lista par nazwa/warto.
	 * @return lista atrybutw rzeczywicie zmienionych
	 *         {@link AttributeList AttributeList}
	 */
	public AttributeList setAttributes(AttributeList attributes) {
		AttributeList setAttributes = new AttributeList();
		for (int a = 0; a < attributes.size(); a++) {
			Attribute attr = (Attribute) attributes.get(a);
			try {
				setAttribute(attr);
				setAttributes.add(attr);
			} catch (Exception ignore) {
			}
		}
		return setAttributes;
	}

	/**
	 * Wywouje operacj na danym zasobie.
	 * 
	 * @param actionName
	 *            nazwa wywoywanej operacji.
	 * @param params
	 *            parametry, ktre zostan przekazane operacji.
	 * @param signature
	 *            sygnatura parametrw.
	 * @return wynik operacji.
	 * @exception MBeanException
	 *                owija bd zgoszony podczas wykonywania operacji.
	 * @exception ReflectionException
	 *                owija bd zgoszony podczas prby odwoania si do
	 *                operacji za pomoc refleksji
	 */
	public Object invoke(String actionName, Object[] params, String[] signature)
			throws MBeanException, ReflectionException {
		Object rtnValue = null;
		log.info("Wywoywana operacja, actionName=" + actionName);
		try {
			if (actionName.equals("echo")) {
				String arg = (String) params[0];
				rtnValue = echo(arg);
				log.info("Wynik: " + rtnValue);
			} else if (actionName.equals("create")) {
				super.create();
			} else if (actionName.equals("start")) {
				super.start();
			} else if (actionName.equals("stop")) {
				super.stop();
			} else if (actionName.equals("destroy")) {
				super.destroy();
			} else {
				throw new JMRuntimeException(
						"Niepoprawny stan podczas wywoywania operacji="
								+ actionName);
			}
		} catch (Exception e) {
			throw new ReflectionException(e, "Bd metody echo");
		}
		log.info("Koniec wywoania, actionName=" + actionName);
		return rtnValue;
	}

	// --- Koniec pokrycia metod interfejsu DynamicMBean

}