package org.jboss.chap2.xmbean;

import java.lang.reflect.Method;
import java.util.HashMap;
import javax.management.Descriptor;
import javax.management.MBeanInfo;

import org.jboss.logging.Logger;
import org.jboss.mx.interceptor.AbstractInterceptor;
import org.jboss.mx.interceptor.Invocation;
import org.jboss.mx.interceptor.InvocationException;
import org.jboss.mx.server.MBeanInvoker;
import org.jboss.invocation.MarshalledInvocation;

/**
 * 
 * @author Scott.Start@jboss.org
 * @version $Revision: 1.5 $
 */

public class InvokerInterceptor extends AbstractInterceptor {
	private static Logger log = Logger.getLogger(InvokerInterceptor.class);
	private Class exposedInterface = ClientInterface.class;
	private HashMap methodMap = new HashMap();
	private HashMap invokeMap = new HashMap();

	public InvokerInterceptor(MBeanInfo info, MBeanInvoker invoker) {
		super(info, invoker);
		try {
			Descriptor[] descriptors = invoker.getDescriptors();
			Object resource = invoker.getResource();

			Class[] getInitialValuesSig = {};
			Method getInitialValues = exposedInterface.getDeclaredMethod(
					"getInitialValues", getInitialValuesSig);
			Long hash = new Long(MarshalledInvocation
					.calculateHash(getInitialValues));
			InvocationInfo invokeInfo = new InvocationInfo("InitialValues",
					Invocation.ATTRIBUTE, Invocation.READ, getInitialValuesSig,
					descriptors, resource);
			methodMap.put(hash, getInitialValues);
			invokeMap.put(getInitialValues, invokeInfo);
			log.debug("warto haszujca getInitialValues: " + hash);

			Class[] setInitialValuesSig = { String[].class };
			Method setInitialValues = exposedInterface.getDeclaredMethod(
					"setInitialValues", setInitialValuesSig);
			hash = new Long(MarshalledInvocation
					.calculateHash(setInitialValues));
			invokeInfo = new InvocationInfo("InitialValues",
					Invocation.ATTRIBUTE, Invocation.WRITE,
					setInitialValuesSig, descriptors, resource);
			methodMap.put(hash, setInitialValues);
			invokeMap.put(setInitialValues, invokeInfo);
			log.debug("warto haszujca setInitialValues: " + hash);

			Class[] getSig = { Object.class };
			Method get = exposedInterface.getDeclaredMethod("get", getSig);
			hash = new Long(MarshalledInvocation.calculateHash(get));
			invokeInfo = new InvocationInfo("get", Invocation.OPERATION,
					Invocation.READ, getSig, descriptors, resource);
			methodMap.put(hash, get);
			invokeMap.put(get, invokeInfo);
			log.debug("warto haszujca get: " + hash);

			Class[] putSig = { Object.class, Object.class };
			Method put = exposedInterface.getDeclaredMethod("put", putSig);
			hash = new Long(MarshalledInvocation.calculateHash(put));
			invokeInfo = new InvocationInfo("put", Invocation.OPERATION,
					Invocation.WRITE, putSig, descriptors, resource);
			methodMap.put(hash, put);
			invokeMap.put(put, invokeInfo);
			log.debug("warto haszujca put: " + hash);
		} catch (Exception e) {
			log.error("Nie powioda si inicjalizacja obiektu InvokerInterceptor",e);
		}
	}

	public Object invoke(Invocation invocation) throws InvocationException {
		String opName = invocation.getName();
		Object[] args = invocation.getArgs();
		Object returnValue = null;
		if (opName.equals("invoke") == true) {
			org.jboss.invocation.Invocation invokeInfo = (org.jboss.invocation.Invocation) args[0];
			// Ustalamy kod haszujcy dla metody
			if (invokeInfo instanceof MarshalledInvocation) {
				MarshalledInvocation mi = (MarshalledInvocation) invokeInfo;
				mi.setMethodMap(methodMap);
			}

			// Wywoanie metody exposedInterface za pomoc refleksji
			Method method = invokeInfo.getMethod();
			Object[] methodArgs = invokeInfo.getArguments();
			InvocationInfo info = (InvocationInfo) invokeMap.get(method);
			Invocation methodInvocation = info.getInvocation(methodArgs);
			returnValue = getNext().invoke(methodInvocation);
		} else {
			returnValue = getNext().invoke(invocation);
		}
		return returnValue;
	}

	/**
	 * Klasa przechowujca informacj o metodzie, wymagana do tego, by przekaza
	 * wywoanie w ramach stosu przechwytywania
	 */
	private class InvocationInfo {
		private int type;
		private int impact;
		private String name;
		private String[] signature;
		private Descriptor[] descriptors;
		private Object resource;

		InvocationInfo(String name, int type, int impact, Class[] signature,
				Descriptor[] descriptors, Object resource) {
			this.name = name;
			this.type = type;
			this.impact = impact;
			this.descriptors = descriptors;
			this.resource = resource;
			this.signature = new String[signature.length];
			for (int s = 0; s < signature.length; s++) {
				this.signature[s] = signature[s].getName();
			}
		}

		Invocation getInvocation(Object[] args) {
			return new Invocation(name, type, impact, args, signature,
					descriptors, resource);
		}

	}

}

