package org.jboss.chap2.xmbean;

import java.security.Principal;

import org.jboss.logging.Logger;
import org.jboss.mx.interceptor.AbstractInterceptor;
import org.jboss.mx.interceptor.Invocation;
import org.jboss.mx.interceptor.InvocationException;
import org.jboss.security.SimplePrincipal;

/**
 * Prosty, odpowiadajcy za bezpieczestwo obiekt przechwytujcy, ktry
 * ogranicza dostp do usugi do jednego uytkownika
 * 
 * @author Scott.Stark@jboss.org
 * @version $Revision: 1.5 $
 */

public class ServerSecurityInterceptor extends AbstractInterceptor {
	private static Logger log = Logger
			.getLogger(ServerSecurityInterceptor.class);

	private SimplePrincipal admin = new SimplePrincipal("admin");

	public String getAdminName() {
		return admin.getName();
	}

	public void setAdminName(String name) {
		admin = new SimplePrincipal(name);
	}

	public Object invoke(Invocation invocation) throws InvocationException {
		String opName = invocation.getName();

		// Pomijane s operacje inne ni invoke(Invocation)
		if (opName.equals("invoke") == false) {
			return getNext().invoke(invocation);
		}

		Object[] args = invocation.getArgs();
		org.jboss.invocation.Invocation invokeInfo = (org.jboss.invocation.Invocation) args[0];
		Principal caller = invokeInfo.getPrincipal();
		log.info("invoke, operacja=" + opName + ", wywoujacy=" + caller);

		// Operacja dozwolona wyacznie dla uytkownika admin
		if (caller == null || caller.equals(admin) == false) {
			throw new InvocationException(new SecurityException("Wywoujcy: "
					+ caller + " nie ma prawa dostpu"));
		}

		return getNext().invoke(invocation);
	}
}