package org.jboss.chap6.ex1;

import javax.jms.JMSException;
import javax.jms.Message;
import javax.jms.MessageListener;
import javax.jms.Topic;
import javax.jms.TopicConnection;
import javax.jms.TopicConnectionFactory;
import javax.jms.TopicPublisher;
import javax.jms.TopicSubscriber;
import javax.jms.TopicSession;
import javax.jms.TextMessage;
import javax.naming.InitialContext;
import javax.naming.NamingException;

/**
 * Program klienta JMS ktry asynchronicznie odbiera komunikaty pojawiajce si
 * w ramach tematu
 * 
 * @author Scott.Stark@jboss.org
 * @version $Revision: 1.4 $
 */
public class DurableTopicRecvClient {
	TopicConnection conn = null;
	TopicSession session = null;
	Topic topic = null;

	public void setupPubSub() throws JMSException, NamingException {
		InitialContext iniCtx = new InitialContext();
		Object tmp = iniCtx.lookup("ConnectionFactory");
		TopicConnectionFactory tcf = (TopicConnectionFactory) tmp;
		conn = tcf.createTopicConnection("john", "needle");
		topic = (Topic) iniCtx.lookup("topic/testTopic");
		session = conn.createTopicSession(false, TopicSession.AUTO_ACKNOWLEDGE);
		conn.start();
	}

	public void recvSync() throws JMSException, NamingException {
		System.out.println("Pocztek metody recvSync");
		// Konfiguracja poczenia typu wydawca-abonent, sesja

		setupPubSub();
		// Czekamy 5 sekund na komunikat
		TopicSubscriber recv = session.createDurableSubscriber(topic,
				"chap6-ex1dtps");
		Message msg = recv.receive(5000);
		if (msg == null) {
			System.out.println("Czas oczekiwania zosta przekroczony");
		} else {
			System.out.println("DurableTopicRecvClient.recv, komunikat=" + msg);
		}
	}

	public void stop() throws JMSException {
		conn.stop();
		session.close();
		conn.close();
	}

	public static void main(String args[]) throws Exception {
		System.out.println("Pocztek DurableTopicRecvClient, czas="
				+ System.currentTimeMillis());
		DurableTopicRecvClient client = new DurableTopicRecvClient();
		client.recvSync();
		client.stop();
		System.out.println("Koniec DurableTopicRecvClient");
		System.exit(0);
	}
}