package org.jboss.chap6.ex1;

import javax.jms.JMSException;
import javax.jms.Message;
import javax.jms.MessageListener;
import javax.jms.Queue;
import javax.jms.QueueConnection;
import javax.jms.QueueConnectionFactory;
import javax.jms.QueueReceiver;
import javax.jms.QueueSender;
import javax.jms.QueueSession;
import javax.jms.TextMessage;
import javax.naming.InitialContext;
import javax.naming.NamingException;

import EDU.oswego.cs.dl.util.concurrent.CountDown;
import org.apache.log4j.Logger;
import org.jboss.util.ChapterExRepository;

/**
 * Kompletny przykad programu klienta JMS, ktry wysya wiadomo tekstow
 * wstawiajc j do kolejki. Nastpnie wiadomo jest asynchronicznie pobierana
 * z kolejki.
 * 
 * @author Scott.Stark@jboss.org
 * @version $Revision: 1.4 $
 */
public class SendRecvClient {
	static Logger log;
	static CountDown done = new CountDown(1);
	QueueConnection conn;
	QueueSession session;
	Queue que;

	public static class ExListener implements MessageListener {
		public void onMessage(Message msg) {
			done.release();
			TextMessage tm = (TextMessage) msg;
			try {
				log.info("onMessage, odebrany tekst=" + tm.getText());
			} catch (Throwable t) {
				t.printStackTrace();
			}
		}
	}

	public void setupPTP() throws JMSException, NamingException {
		InitialContext iniCtx = new InitialContext();
		Object tmp = iniCtx.lookup("ConnectionFactory");
		QueueConnectionFactory qcf = (QueueConnectionFactory) tmp;
		conn = qcf.createQueueConnection();
		que = (Queue) iniCtx.lookup("queue/testQueue");
		session = conn.createQueueSession(false, QueueSession.AUTO_ACKNOWLEDGE);
		conn.start();
	}

	public void sendRecvAsync(String text) throws JMSException, NamingException {
		log.info("Pocztek metody sendRecvAsync");
		// Konfiguracja poczenia typu P2P, sesja
		setupPTP();

		// Konfiguracja asynchronicznego obiektu nasuchujcego
		QueueReceiver recv = session.createReceiver(que);
		recv.setMessageListener(new ExListener());

		// Wysanie komunikatu tekstowego
		QueueSender send = session.createSender(que);
		TextMessage tm = session.createTextMessage(text);
		send.send(tm);
		log.info("sendRecvAsync, wysany tekst=" + tm.getText());
		send.close();
		log.info("Koniec metody sendRecvAsync");
	}

	public void stop() throws JMSException {
		conn.stop();
		session.close();
		conn.close();
	}

	public static void main(String args[]) throws Exception {
		ChapterExRepository.init(SendRecvClient.class);
		log = Logger.getLogger("SendRecvClient");

		log.info("Pocztek SendRecvClient, czas=" + System.currentTimeMillis());
		SendRecvClient client = new SendRecvClient();
		client.sendRecvAsync("Komunikat tekstowy");
		client.done.acquire();
		client.stop();
		log.info("Koniec SendRecvClient");
		System.exit(0);
	}
}