package org.jboss.chap6.ex1;

import javax.jms.JMSException;
import javax.jms.Message;
import javax.jms.MessageListener;
import javax.jms.Topic;
import javax.jms.TopicConnection;
import javax.jms.TopicConnectionFactory;
import javax.jms.TopicPublisher;
import javax.jms.TopicSubscriber;
import javax.jms.TopicSession;
import javax.jms.TextMessage;
import javax.naming.InitialContext;
import javax.naming.NamingException;

/**
 * Przykad klienta JMS, ktry wycznie publikuje komunikat tekstowy typu
 * TextMessage w ramach tematu.
 * 
 * @author Scott.Stark@jboss.org
 * @version $Revision: 1.4 $
 */
public class TopicSendClient {
	TopicConnection conn = null;
	TopicSession session = null;
	Topic topic = null;

	public void setupPubSub() throws JMSException, NamingException {
		InitialContext iniCtx = new InitialContext();
		Object tmp = iniCtx.lookup("ConnectionFactory");
		TopicConnectionFactory tcf = (TopicConnectionFactory) tmp;
		conn = tcf.createTopicConnection();
		topic = (Topic) iniCtx.lookup("topic/testTopic");
		session = conn.createTopicSession(false, TopicSession.AUTO_ACKNOWLEDGE);
		conn.start();
	}

	public void sendAsync(String text) throws JMSException, NamingException {
		System.out.println("Pocztek metody sendAsync");
		// Konfiguracja poczenia typu wydawca-abonent, sesja
		setupPubSub();

		// Wysanie komunikatu tekstowego
		TopicPublisher send = session.createPublisher(topic);
		TextMessage tm = session.createTextMessage(text);
		send.publish(tm);
		System.out.println("sendAsync, wysany tekst=" + tm.getText());
		send.close();
		System.out.println("Koniec metody sendAsync");
	}

	public void stop() throws JMSException {
		conn.stop();
		session.close();
		conn.close();
	}

	public static void main(String args[]) throws Exception {
		System.out.println("Pocztek TopicSendClient, czas="
				+ System.currentTimeMillis());
		TopicSendClient client = new TopicSendClient();
		client.sendAsync("Komunikat tekstowy, czas="
				+ System.currentTimeMillis());
		client.stop();
		System.out.println("Koniec TopicSendClient");
		System.exit(0);
	}
}