package org.jboss.chap6.ex1;

import javax.jms.JMSException;
import javax.jms.Message;
import javax.jms.MessageListener;
import javax.jms.Topic;
import javax.jms.TopicConnection;
import javax.jms.TopicConnectionFactory;
import javax.jms.TopicPublisher;
import javax.jms.TopicSubscriber;
import javax.jms.TopicSession;
import javax.jms.TextMessage;
import javax.naming.InitialContext;
import javax.naming.NamingException;

import EDU.oswego.cs.dl.util.concurrent.CountDown;

/**
 * Kompletny przykad klienta JMS, ktry publikuje komunikat
 * tekstowy typu TextMessage w ramach tematu. Wiadomo ta
 * jest asynchronicznie pobierana.
 * 
 * @author Scott.Stark@jboss.org
 * @version $Revision: 1.4 $
 */
public class TopicSendRecvClient {
	static CountDown done = new CountDown(1);
	TopicConnection conn = null;
	TopicSession session = null;
	Topic topic = null;

	public static class ExListener implements MessageListener {
		public void onMessage(Message msg) {
			done.release();
			TextMessage tm = (TextMessage) msg;
			try {
				System.out.println("onMessage, odebrany tekst=" + tm.getText());
			} catch (Throwable t) {
				t.printStackTrace();
			}
		}
	}

	public void setupPubSub() throws JMSException, NamingException {
		InitialContext iniCtx = new InitialContext();
		Object tmp = iniCtx.lookup("ConnectionFactory");
		TopicConnectionFactory tcf = (TopicConnectionFactory) tmp;
		conn = tcf.createTopicConnection();
		topic = (Topic) iniCtx.lookup("topic/testTopic");
		session = conn.createTopicSession(false, TopicSession.AUTO_ACKNOWLEDGE);
		conn.start();
	}

	public void sendRecvAsync(String text) throws JMSException, NamingException {
		System.out.println("Pocztek metody sendRecvAsync");
		// Konfiguracja poczenia typu wydawca-abonent, sesja
		setupPubSub();

		// Konfiguracja asynchronicznego obiektu nasuchujcego
		TopicSubscriber recv = session.createSubscriber(topic);
		recv.setMessageListener(new ExListener());

		// Wysanie komunikatu tekstowego
		TopicPublisher send = session.createPublisher(topic);
		TextMessage tm = session.createTextMessage(text);
		send.publish(tm);
		System.out.println("sendRecvAsync, wysany tekst=" + tm.getText());
		send.close();
		System.out.println("Koniec metody sendRecvAsync");
	}

	public void stop() throws JMSException {
		conn.stop();
		session.close();
		conn.close();
	}

	public static void main(String args[]) throws Exception {
		System.out.println("Pocztek TopicSendRecvClient, czas="
				+ System.currentTimeMillis());

		TopicSendRecvClient client = new TopicSendRecvClient();
		client.sendRecvAsync("Komunikat tekstowy, czas=" + System.currentTimeMillis());
		client.done.acquire();
		client.stop();
		System.out.println("Koniec TopicSendRecvClient");
		System.exit(0);
	}

}