package org.jboss.chap6.ex2;

import javax.ejb.MessageDrivenBean;
import javax.ejb.MessageDrivenContext;
import javax.ejb.EJBException;
import javax.jms.JMSException;
import javax.jms.Message;
import javax.jms.MessageListener;
import javax.jms.Queue;
import javax.jms.QueueConnection;
import javax.jms.QueueConnectionFactory;
import javax.jms.QueueSender;
import javax.jms.QueueSession;
import javax.jms.TextMessage;
import javax.naming.InitialContext;
import javax.naming.NamingException;

/**
 * Komponent MDB dokonujcy transformacji komunikatw typu TextMessage i
 * wstawiajcy je do kolejki wskazanej w nagwku komunikatu.
 * 
 * @author Scott.Stark@jboss.org
 * @version $Revision: 1.4 $
 */
public class TextMDB implements MessageDrivenBean, MessageListener {
	private MessageDrivenContext ctx = null;

	private QueueConnection conn;

	private QueueSession session;

	public TextMDB() {
		System.out.println("Konstruktor TextMDB, obiekt=" + hashCode());
	}

	public void setMessageDrivenContext(MessageDrivenContext ctx) {
		this.ctx = ctx;
		System.out.println("TextMDB.setMessageDrivenContext, obiekt="
				+ hashCode());
	}

	public void ejbCreate() {
		System.out.println("TextMDB.ejbCreate, obiekt=" + hashCode());
		try {
			setupPTP();
		} catch (Exception e) {
			throw new EJBException("Bd podczas inicjalizacji TextMDB", e);
		}
	}

	public void ejbRemove() {
		System.out.println("TextMDB.ejbRemove, obiekt=" + hashCode());
		ctx = null;
		try {
			if (session != null) {
				session.close();
			}
			if (conn != null) {
				conn.close();
			}
		} catch (JMSException e) {
			e.printStackTrace();
		}
	}

	public void onMessage(Message msg) {
		System.out.println("TextMDB.onMessage, obiekt=" + hashCode());
		try {
			TextMessage tm = (TextMessage) msg;
			String text = tm.getText() + "przetwarzany przez: " + hashCode();
			Queue dest = (Queue) msg.getJMSReplyTo();
			sendReply(text, dest);
		} catch (Throwable t) {
			t.printStackTrace();
		}
	}

	private void setupPTP() throws JMSException, NamingException {
		InitialContext iniCtx = new InitialContext();
		Object tmp = iniCtx.lookup("java:comp/env/jms/QCF");
		QueueConnectionFactory qcf = (QueueConnectionFactory) tmp;
		conn = qcf.createQueueConnection();
		session = conn.createQueueSession(false, QueueSession.AUTO_ACKNOWLEDGE);
		conn.start();
	}

	private void sendReply(String text, Queue dest) throws JMSException {
		System.out.println("TextMDB.sendReply, obiekt=" + hashCode() + ", cel="
				+ dest);
		QueueSender sender = session.createSender(dest);
		TextMessage tm = session.createTextMessage(text);
		sender.send(tm);
		sender.close();
	}

}