package org.jboss.chap8.ex1;

import java.lang.reflect.Method;
import javax.ejb.EJBContext;

import org.apache.log4j.Category;

import org.jboss.security.SecurityProxy;

/** 
 * Prosty przykad niestandardowej implementacji SecurityProxy, 
 * demonstrujcy testy bezpieczestwa wykonywane w oparciu o argumenty metod.
 *
 * @author Scott.Stark@jboss.org
 * @version $Revision: 1.2 $
 */
public class EchoSecurityProxy 
    implements SecurityProxy
{
    Category log = Category.getInstance(EchoSecurityProxy.class);
    Method echo;
    
    public void init(Class beanHome, Class beanRemote,
                     Object securityMgr)
        throws InstantiationException
    {
        init(beanHome, beanRemote, null, null, securityMgr);
    }

    public void init(Class beanHome, Class beanRemote,
                     Class beanLocalHome, Class beanLocal, Object securityMgr)
        throws InstantiationException
    {
        log.debug("init, beanHome=" + beanHome + ", beanRemote=" + beanRemote +
                  ", beanLocalhome=" + beanLocalHome + ", beanLocal=" + beanLocal +
                  ", securityMgr=" + securityMgr);
        // Pobranie metody echo na potrzeby sprawdzenia rwnoci podczas wywoania
        try {
            Class[] params = {String.class};
            echo = beanRemote.getDeclaredMethod("echo", params);
        } catch(Exception e) {
            String msg = "Nie udao si odnale metody echo(String)";
            log.error(msg, e);
            throw new InstantiationException(msg);
        }
    }

    public void setEJBContext(EJBContext ctx)
    {
        log.debug("setEJBContext, ctx=" + ctx);
    }

    public void invokeHome(Method m, Object[] args)
        throws SecurityException
    {
        // Nie weryfikujemy dotpu do metod bazowych
    }

    public void invoke(Method m, Object[] args, Object bean)
        throws SecurityException
    {
        log.debug("invoke, m=" + m);
        // Sprawdzenie metody echo
        if (m.equals(echo)) {
            // Sprawdzenie czy argument ma dugo 4 znakw
            String arg = (String) args[0];
            if (arg == null || arg.length() == 4) {
                throw new SecurityException("No 4 letter words");
            }
            // To my jestemy odpowiedzialni za wywoanie
        }
    }
}
