package org.jboss.chap8.ex2;

import java.security.acl.Group;
import java.util.Map;
import javax.naming.InitialContext;
import javax.naming.NamingException;
import javax.security.auth.Subject;
import javax.security.auth.callback.CallbackHandler;
import javax.security.auth.login.LoginException;

import org.jboss.security.SimpleGroup;
import org.jboss.security.SimplePrincipal;
import org.jboss.security.auth.spi.UsernamePasswordLoginModule;

/** 
 * Przykad niestandardowego moduu logowania, ktry pobiera
 * haso oraz role uytkownika przy wykorzystaniu wysukiwania JNDI.
 *
 * @author Scott.Stark@jboss.org
 * @version $Revision: 1.1 $
 */
 
public class JndiUserAndPass extends UsernamePasswordLoginModule
{
    /** Nazwa JNDI okrelajca kontekst obsugujcy wyszukiwanie password/username */
    private String userPathPrefix;
    /** Nazwa JNDI okrelajca kontekst obsugujcy wyszukiwanie roles/username */
    private String rolesPathPrefix;

    /**
     * T metod naley przesoni w celu pobrania wartoci opcji userPathPrefix 
     * oraz rolesPathPrefix
     */
   public void initialize(Subject subject, CallbackHandler callbackHandler,
      Map sharedState, Map options)
   {
      super.initialize(subject, callbackHandler, sharedState, options);
      userPathPrefix = (String) options.get("userPathPrefix");
      rolesPathPrefix = (String) options.get("rolesPathPrefix");
   }

    /**
     * Metoda pobiera role do jakich naley biecy uytkownik poprzez
     * wyszukanie w kontekcie JNDI acucha
     * rolesPathPrefix + '/' + super.getUsername().
     */
   protected Group[] getRoleSets() throws LoginException
   {
      try
      {
         InitialContext ctx = new InitialContext();
         String rolesPath = rolesPathPrefix + '/' + super.getUsername();
         String[] roles = (String[]) ctx.lookup(rolesPath);
         Group[] groups = {new SimpleGroup("Roles")};
         log.info("Pobieranie rl dla uytkownika="+super.getUsername());
         for(int r = 0; r < roles.length; r ++)
         {
            SimplePrincipal role = new SimplePrincipal(roles[r]);
            log.info("Odnaleziono rol="+roles[r]);
            groups[0].addMember(role);
         }
         return groups;
      }
      catch(NamingException e)
      {
         log.error("Nie udao si pobra grup dla uytkownika="+super.getUsername(), e);
         throw new LoginException(e.toString(true));
      }
   }

    /**
     * Metoda pobiera haso biecego uytkownik poprzez
     * wyszukanie w kontekcie JNDI acucha
     * userPathPrefix + '/' + super.getUsername().
     */
   protected String getUsersPassword() throws LoginException
   {
      try
      {
         InitialContext ctx = new InitialContext();
         String userPath = userPathPrefix + '/' + super.getUsername();
         log.info("Pobieranie hasa uytkownika="+super.getUsername());
         String passwd = (String) ctx.lookup(userPath);
         log.info("Odnaleziono haso="+passwd);
         return passwd;
      }
      catch(NamingException e)
      {
         log.error("Nie udao si odczyta hasa dla uytkownika="+super.getUsername(), e);
         throw new LoginException(e.toString(true));
      }
   }

}
