package org.jboss.chap8.ex2.service;

import java.lang.reflect.InvocationHandler;
import java.lang.reflect.Method;
import java.lang.reflect.Proxy;
import java.util.Hashtable;
import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.InvalidNameException;
import javax.naming.Name;
import javax.naming.NameParser;
import javax.naming.NamingException;
import javax.naming.OperationNotSupportedException;
import javax.naming.RefAddr;
import javax.naming.Reference;
import javax.naming.StringRefAddr;
import javax.naming.spi.ObjectFactory;

import org.apache.log4j.Logger;
import org.jboss.naming.Util;

/**
 *
 * @author Scott.Stark@jboss.org
 * @version $Revision: 1.1 $
 */
public class JndiStore
   implements JndiStoreMBean, InvocationHandler, ObjectFactory
{
   private static Logger log = Logger.getLogger(JndiStore.class);

   private static String password = "theduke";
   private static String[] roles = {"TheDuke", "Echo"};
   private static NameParser parser;

   /** Tworzy now instacj obiektu JndiStore */
   public JndiStore()
   {
   }

   public void start() throws Exception
   {
      InitialContext ctx = new InitialContext();
      parser = ctx.getNameParser("");

      /* 
       Utworzenie odwzorowania pomidzy kontekstem security/store a tym obiektem
       ObjectFactory, dziki czemu wszelkie wyszukiwanie przy uyyciu nazwy
       security/store/haso/nazwaUytkownika nasze haso oraz 
       security/store/roles/nazwaUytkownika zwrci nasze role.
      */
      RefAddr refAddr = new StringRefAddr("nns", "JndiStore");
      String factoryName = JndiStore.class.getName();
      Reference ref = new Reference("javax.naming.Context", refAddr, factoryName, null);
      
      Util.rebind(ctx, "security/store", ref);
      log.info("Pocztek, powizano security/store");
   }
   public void stop()
   {
      try
      {
         InitialContext ctx = new InitialContext();
         ctx.unbind("security/store");
         log.info("Koniec, usunito powizanie security/store");
      }
      catch(NamingException e)
      {
      }
   }

   /** Implementacja fabryki obiektw. Ta metoda zwraca porednika Context
    * umoliwiajcego wykonywanie jedynie operacji wyszukiwania atomowych nazw
    * dziedziny bezpieczestwa.
   */
   public Object getObjectInstance(Object obj, Name name, Context nameCtx, Hashtable environment)
      throws Exception
   {
      ClassLoader loader = Thread.currentThread().getContextClassLoader();
      Class[] interfaces = {Context.class};
      Context ctx = (Context) Proxy.newProxyInstance(loader, interfaces, this);
      return ctx;
   }

   /** To jest metoda zwrotna InvocationHandler interfejsu Context, utworzona przez 
    * metod getObjectInstance(). W tym miejscu jest obsugiwane przeszukiwanie
    * kontekstu security/store.
    */
   public Object invoke(Object obj, Method method, Object[] args) throws Throwable
   {
      String methodName = method.getName();
      if( methodName.equals("toString") == true )
         return "Porednik kontekstu security/store";
      if( methodName.equals("lookup") == false )
         throw new OperationNotSupportedException("Dostpna jest jedynie operacja lookup, op="+method);

      Name name = null;
      if( args[0] instanceof String )
         name = parser.parse((String) args[0]);
      else
         name = (Name) args[0];
      String type = name.toString();
      log.info("wyszukiwanie, nazwa="+name);
      if( type.startsWith("password") == true )
         return password;
      if( type.startsWith("roles") == true )
         return roles;
      throw new InvalidNameException("danie musi by typu password lub roles, typ="+type);
   }
}
