package org.jboss.chap8.ex3.service;

import java.io.FileNotFoundException;
import java.io.InputStream;
import java.io.IOException;
import java.math.BigInteger;
import java.net.URL;
import java.security.KeyException;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Properties;
import javax.naming.InitialContext;
import javax.naming.Name;

import org.jboss.naming.NonSerializableFactory;
import org.jboss.security.Util;
import org.jboss.security.srp.SRPConf;
import org.jboss.security.srp.SRPVerifierStore;
import org.jboss.security.srp.SRPVerifierStore.VerifierInfo;
import org.jboss.system.ServiceMBeanSupport;

/** Usuga PropertiesVerifierStore stanowi implementacj SRPVerifierStore 
 ktra pobiera informacje o nazwi uytkownika i hale z pliku waciwoci 
 a nastpnie tworzy w pamici SRPVerifierStore.

@author Scott.Stark@jboss.org
@version $Revision: 1.1 $
*/
public class PropertiesVerifierStore extends ServiceMBeanSupport
   implements PropertiesVerifierStoreMBean, SRPVerifierStore
{
   private String jndiName = "srp/DefaultVerifierSource";
   private HashMap storeMap = new HashMap();
   private Thread addUserThread;

   /** Utworzenie nowej instancji PropertiesVerifierStore */
   public PropertiesVerifierStore()
   {
   }

   /** Pobranie nazwy jndi na potrzeby powizania implementacji SRPVerifierSource.
   */
   public String getJndiName()
   {
     return jndiName;
   }
   /** Okrelenie nazwy jndi na potrzeby powizania implementacji SRPVerifierSource.
   */
   public void setJndiName(String jndiName)
   {
      this.jndiName = jndiName;
   }

   protected void startService() throws Exception
   {
      // Upewniamy si czy zostaa zainicjowana klasa pomocnicza
      Util.init();

      // Odnajdujemy plik users.properties
      ClassLoader loader = Thread.currentThread().getContextClassLoader();
      URL users = loader.getResource("users.properties");
      if( users == null )
         throw new FileNotFoundException("Nie udao si odnale zasobu users.properties");
      log.info("Uywamy users.properties: "+users);
      InputStream is = users.openStream();
      final Properties userPasswords = new Properties();
      userPasswords.load(is);
      addUserThread = new Thread("AddUsers")
      {
         public void run()
         {
            Iterator keys = userPasswords.keySet().iterator();
            while( keys.hasNext() )
            {
               String username = (String) keys.next();
               char[] password = userPasswords.getProperty(username).toCharArray();
               addUser(username, password);
            }
         }
      };
      addUserThread.start();

      // Powizanie referencji do SRPVerifierStore przy uyciu klasy NonSerializableFactory
      InitialContext ctx = new InitialContext();
      Name name = ctx.getNameParser("").parse(jndiName);
      NonSerializableFactory.rebind(name, this, true);
      log.debug("Powizano SRPVerifierStore uywajc nazwy "+jndiName);
   }
   protected void stopService() throws Exception
   {
      InitialContext ctx = new InitialContext();
      NonSerializableFactory.unbind(jndiName);
      ctx.unbind(jndiName);
      log.debug("Usunito powizanie SRPVerifierStore uywajce nazwy "+jndiName);
   }

   public VerifierInfo getUserVerifier(String username) throws KeyException, IOException
   {
      if( addUserThread != null )
      {
         try
         {
            addUserThread.join();
            addUserThread = null;
         }
         catch(InterruptedException e)
         {
         }
      }
      VerifierInfo info = (VerifierInfo) storeMap.get(username);
      return info;
   }
   public void setUserVerifier(String username, VerifierInfo info) throws IOException
   {
      throw new IOException("Zasb PropertiesVerifierStore jest przeznaczony tylko do odczytu");
   }

   public void verifyUserChallenge(String username, Object challenge)
   {
   }

   private void addUser(String username, char[] password)
   {
      VerifierInfo info = new VerifierInfo();
      info.username = username;
      // Utworzenie losowych danych inicjalizacyjnych
      long r = Util.nextLong();
      String rs = Long.toHexString(r);
      info.salt = rs.getBytes();
      BigInteger g = SRPConf.getDefaultParams().g();
      BigInteger N = SRPConf.getDefaultParams().N();

      info.verifier = Util.calculateVerifier(username, password, info.salt, N, g);
      info.g = g.toByteArray();
      info.N = N.toByteArray();
      log.info("Dodano uytkownika: "+username);
      storeMap.put(username, info);
   }
}
