/*
* JBoss, the OpenSource J2EE webOS
*
* Distributable under LGPL license.
* See terms of license at gnu.org.
*/
package org.jboss.chap8.ex3a.service;

import java.lang.reflect.Method;
import java.io.Serializable;
import java.net.InetAddress;
import java.util.Vector;
import javax.naming.InitialContext;
import javax.naming.Name;

import org.jboss.ha.framework.interfaces.DistributedState;
import org.jboss.ha.framework.interfaces.DistributedState.DSListenerEx;
import org.jboss.ha.framework.interfaces.HAPartition;
import org.jboss.ha.framework.interfaces.HAPartition.HAMembershipListener;
import org.jboss.ha.framework.server.util.DistributedTimedCachePolicy;
import org.jboss.naming.NonSerializableFactory;
import org.jboss.system.ServiceMBeanSupport;

/** Mbean implementujcy monitor czonkostwa partycji klastra 
 * oraz test rozproszonej pamici podrcznej.

 @author Scott.Stark@jboss.org
 @version $Revision: 1.1 $
 */
public class DistributedCacheService extends ServiceMBeanSupport
   implements DistributedCacheServiceMBean, HAMembershipListener, DSListenerEx
{
   private String jndiName;
   private String partitionName = "Default";
   private HAPartition partition;
   private DistributedState map;
   private DistributedTimedCachePolicy cache;
   private int cacheTimeout = 1800;

// --- Pocztek przesonitych metod ServiceMBeanSupport 
   protected void startService() throws Exception
   {
      InitialContext ctx = new InitialContext();
      String partitionJndiName = "/HAPartition/" + partitionName;
      partition = (HAPartition) ctx.lookup(partitionJndiName);
      map = partition.getDistributedStateService();
      cache = new DistributedTimedCachePolicy(jndiName, partitionName, cacheTimeout);
      cache.create();
      cache.start();
      // Powizanie pamici podrcznej z JNDI
      Name name = ctx.getNameParser("").parse(jndiName);
      NonSerializableFactory.rebind(name, cache, true);

      // Rejestracja odbiorcy zdarze zwizanych ze zmianami pamici podrcznej
      map.registerDSListenerEx(jndiName, this);
      log.info("Zarejestrowany jako DMListener w kategorii="+jndiName);
      // Rejestracja odbiorcy zdaze zwizanych ze zmianami czonkostwa klastra
      partition.registerMembershipListener(this);
      log.info("Zarejestrowany jako MembershipListener");
   }

   protected void stopService() throws Exception
   {
      cache.stop();
      map.unregisterDSListenerEx(jndiName, this);
      partition.unregisterMembershipListener(this);
   }
// --- Koniec przesonitych metod ServiceMBeanSupport

// --- Pocztek metod interfejsu DistributedCacheServiceMBean 
   public String getPartitionName()
   {
      return partitionName;
   }
   public void setPartitionName(String name)
   {
      this.partitionName = name;
   }

   public String getCacheJndiName()
   {
      return jndiName;
   }
   public void setCacheJndiName(String name)
   {
      this.jndiName = name;
   }

   public int getCacheTimeout()
   {
      return cacheTimeout;
   }
   public void setCacheTimeout(int timeoutSecs)
   {
      this.cacheTimeout = timeoutSecs;
   }

   public Vector getClusterNodes()
   {
      Vector view = null;
      try
      {
         InitialContext ctx = new InitialContext();
         String jndiName = "/HAPartition/" + partitionName;
         HAPartition partition = (HAPartition) ctx.lookup(jndiName);
         view = partition.getCurrentView();
      }
      catch(Exception e)
      {
         log.error("Nie udao si uzyska dostpu do stanu HAPartition", e);
      }
      return view;
   }

   public Object get(String key)
   {
      return cache.get(key);
   }
   public void set(String key, String value)
   {
      cache.insert(key, value);
   }
// --- Koniec metod interfejsu DistributedCacheServiceMBean

// --- Pocztek metod interfejsu HAMembershipListener 
   /** Metoda wywoywana w momencie pojawienia si nowej topologii partycji.
    * @param deadMembers Lista wzw, ktre od ostatniego przegldy stay si nieaktywne
    * @param newMembers Lista wzw, ktre od ostatniego przegldu zostay dodane do partycji
    * @param allMembers Lista wzw tworzcych biecy widok partycji
    */
   public void membershipChanged(Vector deadMembers, Vector newMembers, Vector allMembers)
   {
      log.info("DeadMembers: ilo="+deadMembers.size());
      for(int m = 0; m < deadMembers.size(); m ++)
      {
         AddressPort addrInfo = getMemberAddress(deadMembers.get(m));
         log.info(addrInfo);
      }
      log.info("NewMembers: ilo="+newMembers.size());
      for(int m = 0; m < newMembers.size(); m ++)
      {
         AddressPort addrInfo = getMemberAddress(newMembers.get(m));
         log.info(addrInfo);
      }
      log.info("AllMembers: ilo="+allMembers.size());
      for(int m = 0; m < allMembers.size(); m ++)
      {
         AddressPort addrInfo = getMemberAddress(allMembers.get(m));
         log.info(addrInfo);
      }
   }
// --- Koniec metod interfejsu HAMembershipListener interface methods

// --- Pocztek metod interfejsu DMListener 
   public void valueHasChanged(String category, Serializable key,
      Serializable value, boolean locallyModified)
   {
      log.info("valueHasChanged, category="+category+", key="+key
         +", value="+value+", locallyModified="+locallyModified);
   }
   public void keyHasBeenRemoved (String category, Serializable key,
      Serializable value, boolean locallyModified)
   {
      log.info("keyHasBeenRemoved, category="+category+", key="+key
         +", value="+value+", locallyModified="+locallyModified);
   }
// --- Koniec metod interfejsu DMListener

   /** Metoda uywa refleksji do pobrania adresu InetAddress oraz portu 
    * jeli tyko s one dostpne w przekazanej implementacji interfejsu Address 
    */
   private AddressPort getMemberAddress(Object addr)
   {
      AddressPort info = null;
      try
      {
         Class[] parameterTypes = {};
         Object[] args = {};
         Method getIpAddress = addr.getClass().getMethod("getIpAddress", parameterTypes);
         InetAddress inetAddr = (InetAddress) getIpAddress.invoke(addr, args);
         Method getPort = addr.getClass().getMethod("getPort", parameterTypes);
         Integer port = (Integer) getPort.invoke(addr, args);
         info = new AddressPort(inetAddr, port);
      }
      catch(Exception e)
      {
         // Prbujemy pobra informacje o adresie przy uyciu metody 
         //  getName -> addr:port 
         try
         {
            Class[] parameterTypes = {};
            Object[] args = {};
            Method getName = addr.getClass().getMethod("getName", parameterTypes);
            String id = (String) getName.invoke(addr, args);
            String addrString = id;
            Integer port = null;
            int colon = id.indexOf(':');
            if( colon > 0 )
            {
               addrString = id.substring(0, colon);
               port = Integer.valueOf(id.substring(colon+1));
            }
            InetAddress inetAddr = InetAddress.getByName(addrString);
            info = new AddressPort(inetAddr, port);
         }
         catch(Exception e2)
         {
            log.warn("Nie udao si pobra informacji InetAddress/port z adresu: "+addr, e);            
         }
      }
      return info;
   }

   static class AddressPort
   {
      InetAddress addr;
      Integer port;
      AddressPort(InetAddress addr, Integer port)
      {
         this.addr = addr;
         this.port = port;
      }
      public String toString()
      {
         return "{host("+addr+"), port("+port+")}";
      }
   }
}
