package org.jboss.chap8.service;

import java.net.URL;
import javax.management.MBeanServer;
import javax.management.ObjectName;

import org.jboss.mx.util.MBeanProxy;
import org.jboss.security.auth.login.XMLLoginConfigMBean;
import org.jboss.system.ServiceMBeanSupport;

/** 
 Komponent mbean konfiguracji bezpieczestwa, ktry wczytuje konfiguracj
 logowania zapisan w formie kodu XML i dodaje instacj XMLLoginConfig
 do stosu zarzdzanego przez mbean SecurityConfigName 
 (default=jboss.security:service=XMLLoginConfig).
 
 @author Scott.Stark@jboss.org
 @version $Revision: 1.1 $
 */
public class SecurityConfig extends ServiceMBeanSupport
   implements SecurityConfigMBean
{
   // Stae -----------------------------------------------------
   
   // Atrybuty ----------------------------------------------------
   private String authConf = "login-config.xml";
   private String[] loadedDomains;
   private ObjectName mainSecurityConfig;
   private XMLLoginConfigMBean config;

   // Skadowe statyczne --------------------------------------------------------

   // Konstruktory --------------------------------------------------
   public SecurityConfig()
   {
      setSecurityConfigName("jboss.security:service=XMLLoginConfig");
   }

   public String getName()
   {
      return "Konfiguracja logowania JAAS";
   }
   public String getSecurityConfigName()
   {
      return mainSecurityConfig.toString();
   }
   public void setSecurityConfigName(String objectName)
   {
      try
      {
         mainSecurityConfig = new ObjectName(objectName);
      }
      catch(Exception e)
      {
         log.error("Nie udao si utworzy obiektu ObjectName", e);
      }
   }

   /** Pobiera ciek zasobu okrelajc pooenie pliku konfiguracji logowania JAAS,
    *  ktry ma by uywany.
    */
   public String getAuthConfig()
   {
      return authConf;
   }

   /** Ustawia ciek zasobu okrelajc pooenie pliku konfiguracji logowania JAAS,
    *  ktry ma by uywany. Domylnie jest to "login-config.xml".
    */
   public void setAuthConfig(String authConf)
   {
      this.authConf = authConf;
   }

   /** Metoda uruchamia dziaanie usugi. Operacja ta wymaga odszukania zasobu
    * AuthConfig oraz zaadowania go do komponentu XMLLoginConfigMBean
    * okrelonego przez mbean SecurityConfigName.
    */
   protected void startService() throws Exception
   {
      // Odszukanie authConf jako zasobu
      ClassLoader loader = Thread.currentThread().getContextClassLoader();
      URL loginConfig = loader.getResource(authConf);
      if( loginConfig != null )
      {
         log.info("Using JAAS AuthConfig: "+loginConfig.toExternalForm());
         config = (XMLLoginConfigMBean) MBeanProxy.get(XMLLoginConfigMBean.class,
            mainSecurityConfig, server);
         loadedDomains = config.loadConfig(loginConfig);
      }
      else
      {
         log.warn("Nie udao si odszuka zasobu AuthConfig");
      }
   }

   protected void stopService() throws Exception
   {
      if( loadedDomains != null )
         config.removeConfigs(loadedDomains);
   }
}
