package org.jboss.cmp2.crimeportal;

import java.io.Serializable;

/**
 * Klasa ContactInfo zawiera numer telefonu komrkowego, numer pagera oraz 
 * adres poczty elektronicznej.
 */
public class ContactInfo 
    implements Serializable 
{
    /** Numer telefonu komrkowego. */
    private PhoneNumber cell;
    
    /** Numer pagera. */
    private PhoneNumber pager;
    
    /** Adres poczty elektronicznej. */
    private String email;
    
    /**
     * Konstruktor tworzy pusty obiekt ContactInfo.
     */
    public ContactInfo() {
    }
    
    /**
     * Odczyt numeru telefonu komrkowego.
     * @return numer telefonu komrkowego.
     */
    public PhoneNumber getCell() {
        return cell;
    }
    
    /**
     * Podanie numeru telefonu komrkowego.
     * @param nowy numer telefonu komrkowego.
     */
    public void setCell(PhoneNumber cell) {
        this.cell = cell;
    }
    
    /**
     * Odczyt numeru pagera.
     * @return numer pagera.
     */
    public PhoneNumber getPager() {
        return pager;
    }
    
    /**
     * Podanie numeru pagera.
     * @param nowy numer pagera.
     */
    public void setPager(PhoneNumber pager) {
      this.pager = pager;
    }
    
    /**
     * Odczyt adres poczty elektronicznej.
     * @return adres poczty elektronicznej.
     */
    public String getEmail() {
        return email;
    }
    
    /**
     * Podanie numeru poczty elektronicznej.
     * @param nowy adres poczty elektronicznej
     */
    public void setEmail(String email) {
        this.email = email.toLowerCase();
    }
    
    public boolean equals(Object o) {
        if (o == this) {
            return true;
        }
        if (!(o instanceof ContactInfo)) {
            return false;
        }
        ContactInfo contactInfo = (ContactInfo)o;
        return isEqual(contactInfo.cell, cell) &&
            isEqual(contactInfo.pager, pager) &&
            isEqual(contactInfo.email, email);
    }
   
    /**
     * Pomocniczna metoda sprawdzajca rwno i poprawnie obsugujca
     * wartoci null.
     * @param o1 pierwszy z porwnywanych obiektw
     * @param o2 drugi z porwnywanych obiektw
     * @return true jeli oba obiekty s rwne null lub s swymi odpowiednikami; false 
     * w pozotaych przypadkach.
     */
    private static boolean isEqual(Object o1, Object o2) {
      if (o1 == null) {
          return o2 == null;
      }
      return o1 == o2 || o1.equals(o2);
   }
   
    public int hashCode() {
        int result = 17;    
        result = 37 * result + (cell==null ? 0 : cell.hashCode());
        result = 37 * result + (pager==null ? 0: pager.hashCode());
        result = 37 * result + (email==null ? 0 : email.hashCode());
        return result;
    }
    
    /**
     * Zwraca informacje o kontakcie w postaci jednego acucha znakw.
     * Szczegy reprezentacji nie s okrelone i mog si zmienia,
     * jednak mona uzna, e typowa reprezentacja bdzie mie nastpujc
     * posta:
     *
     * "[ContactInfo: komrka=(123) 456-7890, pager=(123) 456-7890,
     *    email=someone@somewhere.com]"
     */
    public String toString() {
        return "[ContactInfo: " + 
            "komrka=" + cell + ", " +
            "pager=" + pager + ", " +
            "email=" + email + "]";
    }
}
