package org.jboss.cmp2.crimeportal;

import javax.naming.InitialContext;

import junit.framework.Test;
import junit.framework.TestSuite;
import org.jboss.test.util.ejb.EJBTestCase;

public class CrimePortalSetUp extends EJBTestCase
   implements CrimePortalTestConstants
{

   public static Test suite()
   {
      TestSuite testSuite = new TestSuite("CrimePortalSetUp");
      testSuite.addTestSuite(CrimePortalSetUp.class);
      return testSuite;
   }

   public CrimePortalSetUp(String name)
   {
      super(name);
   }

   /**
    * Przeglda wszystkie interfejsy bazowe i tworzy dane pocztkowe.
    * @throws Wyjtek jeli podczas poszukiwania interfejsw bazowych 
    * wystpi bdy, bd te jeli pojawi si problemy podczas tworzenia
    * danych pocztkowych.
    */
   public void testSetUp() throws Exception
   {
      InitialContext jndi = new InitialContext();

      OrganizationHome organizationHome =
         (OrganizationHome) jndi.lookup("crimeportal/Organization");

      GangsterHome gangsterHome =
         (GangsterHome) jndi.lookup("crimeportal/Gangster");

      JobHome jobHome = (JobHome) jndi.lookup("crimeportal/Job");

      LocationHome locationHome =
         (LocationHome) jndi.lookup("crimeportal/Location");

      // Utworzenie kilku organizacji
      Organization yakuza =
         organizationHome.create("Yakuza", "Gangsterzy z Japonii");
      Organization mafia =
         organizationHome.create("Mafia", "li faceci z Woch");
      Organization triads =
         organizationHome.create("Triads", "Dodatek do filmw Kung Fu");

      // Utworzenie kilku gangsterw
      Gangster yojimbo =
         gangsterHome.create(YOJIMBO, "Yojimbo", "Ochronka", 7, yakuza);
      ContactInfo contactInfo = new ContactInfo();
      contactInfo.setEmail("yojimbo439@yakuza.jp");
      contactInfo.setCell(new PhoneNumber(123, 456, 7890));
      contactInfo.setPager(new PhoneNumber(111, 222, 3333));
      yojimbo.setContactInfo(contactInfo);
      yojimbo.setHangout(locationHome.create("Piwnica Pod Czerwonym Smokiem",
         "1452 Stockton Street", "San Fran", "CA", 94108));

      Gangster takeshi =
         gangsterHome.create(TAKESHI, "Takeshi", "Mistrzu", 10, yakuza);
      takeshi.setHangout(locationHome.create("Kafeja Poncej Pici",
         "598 Jackson Street", "San Fran", "CA", 94133));
      Gangster yuriko =
         gangsterHome.create(YURIKO, "Yuriko", "Czteropalczasty", 4, yakuza);
      yuriko.setHangout(locationHome.create("Dom Siostry",
         "1411 Powell Street", "San Fran", "CA", 94133));

      Gangster chow =
         gangsterHome.create(CHOW, "Chow", "Zabjca", 9, triads);
      chow.setHangout(locationHome.create("Fabryka Szczliwych ciasteczak Golden Gate",
         "56 Ross Alley", "San Fran", "CA", 94133));
      Gangster shogi =
         gangsterHome.create(SHOGI, "Shogi", "Byskawica", 8, triads);
      shogi.setHangout(locationHome.create("Sklep Wok",
         "718 Grant Avenue", "San Fran", "CA", 94133));

      Gangster valentino =
         gangsterHome.create(VALENTINO, "Valentino", "Pizzernik", 4, mafia);
      valentino.setHangout(locationHome.create("U Luki",
         "299 Madison Avenue", "New York", "NY", 10017));
      Gangster toni =
         gangsterHome.create(TONI, "Toni", "Bezzbny", 2, mafia);
      toni.setHangout(locationHome.create("Hote Unii Pracownikw Transportowych",
         "225 S. Canal Street", "Chicago", "IL", 60661));
      Gangster corleone =
         gangsterHome.create(CORLEONE, "Corleone", "Ojciec Chrzestny", 6, mafia);
      corleone.setHangout(locationHome.create("Kasyno - Czarny Diament",
         "9555 Las Vegas Blvd South", "Las Vegas", "NV", 89109));

      // Okrelenie szefw organizacji
      yakuza.setTheBoss(takeshi);
      triads.setTheBoss(chow);
      mafia.setTheBoss(corleone);

      // Utworzenie kilku zada
      Job jewler = jobHome.create("Napad na jubilera na 10-tej ulicy");
      jewler.setScore(5000);
      jewler.setSetupCost(50);

      Job train = jobHome.create("Wielki skok na pocig");
      train.setScore(2000000);
      train.setSetupCost(500000);

      Job liquorStore = jobHome.create("Wam do sklepu z tanimi alkoholami");
      liquorStore.setScore(50);
      liquorStore.setSetupCost(0);

      // przypisanie gangsterw do zada
      jewler.getGangsters().add(valentino);
      jewler.getGangsters().add(corleone);

      train.getGangsters().add(yojimbo);
      train.getGangsters().add(chow);

      liquorStore.getGangsters().add(chow);
   }
}
