package org.jboss.cmp2.crimeportal;

import javax.ejb.CreateException;
import javax.ejb.EntityBean;
import javax.ejb.EntityContext;

import org.apache.log4j.Category;

public abstract class LocationBean implements EntityBean
{
   private EntityContext ctx;
   private Category log = Category.getInstance(getClass());

   public Integer ejbCreate() throws CreateException
   {
      return null;
   }

   public void ejbPostCreate()
   {
      Object id = ctx.getPrimaryKey();
      log.debug("ejbPostCreate, id="+id);
   }

   public Integer ejbCreate(String description, String street, String city,
         String state, int zipCode) throws CreateException
   {
      setDescription(description);
      setStreet(street);
      setCity(city);
      setState(state);
      setZipCode(zipCode);
      return null;
   }

   public void ejbPostCreate(String description, String street, String city, 
         String state, int zipCode) throws CreateException
   {
      Object id = ctx.getPrimaryKey();
      log.debug("ejbPostCreate, id="+id);
   }

   public abstract Integer getLocationID();
   public abstract void setLocationID(Integer id);

   /**
    * Odczyt opisu adresu.
    * @return opis adresu
    */
   public abstract String getDescription();

   /**
    * Podanie opisu adresu.
    * @param description nowy opis adresu
    */
   public abstract void setDescription(String description);

   /**
    * Odczyt nazwy ulicy z adresu.
    * @return street nazwa ulicy
    */
   public abstract String getStreet();

   /**
    * Podanie nazwy ulicy.
    * @param street nowa nazwa ulicy
    */
   public abstract void setStreet(String street);
 
   /**
    * Odczyt miasta z adresu.
    * @return nazwa miasta
    */
   public abstract String getCity();

   /**
    * Podanie nazwy miasta.
    * @param city nowa nazwa miasta
    */
   public abstract void setCity(String city);

   /**
    * Odczyt dwuliterowego kodu stanu z adresu
    * @return dwuliterowy kod stanu
    */
   public abstract String getState();

   /**
    * Podanie dwuliterowego kodu stanu.
    * @param state nowy dwuliterowy kod stanu
    * @throws IllegalArgumentException jeli okrelenie stanu nie skasa
    * si dokadnie z dwch liter
    */
   public abstract void setState(String state);
   
   /**
    * Odczyt kodu pocztowego z adresu.
    * @return kod pocztowy
    */
   public abstract int getZipCode();

   /**
    * Okrelenie nowego kodu pocztowego.
    * @param zipCode nowy kod pocztowy
    * @throws IllegalArgumentException jeli kod pocztowy jest liczb ujemn
    * lub jeli zawiera wicej ni 5 cyfr.
    */
   public abstract void setZipCode(int zipCode);

   // Metody zwrotne EJB -----------------------------------------------------------
   public void setEntityContext(EntityContext context)
   {
      ctx = context;
   }

   public void unsetEntityContext()
   {
      ctx = null;
   }

   public void ejbActivate()
   {
   }

   public void ejbPassivate()
   {
   }

   public void ejbRemove()
   {
   }

   public void ejbStore()
   {
   }

   public void ejbLoad()
   {
   }
}
