package org.jboss.cmp2.crimeportal;

import java.io.Serializable;

/**
 * PhoneNumber zawiera numery telefoniczne stosowane w USA podzielone na trzy
 * odrbne fragmenty.
 */
public class PhoneNumber 
    implements Serializable 
{
    /** Pierwsze trzy cyfry numeru telefonu. */
    private short areaCode;
    
    /** rodkowe trzy cyfry numeru telefonu. */
    private short exchange;
    
    /** Ostatnie cztery cyfry numeru telefonu. */
    private short extension;
    
    /**
     * Tworzy numer o postaci (000) 000-0000.
     */
    public PhoneNumber() {
    }
    
    /**
     * Tworzy numer telefonicznych skadajcy si z podanych elementw areaCode, 
     * exchange oraz extension.
     * @param areaCode pierwsze trzy cyfry numeru
     * @param exchange rodkowe trzy cyfry numeru
     * @param extension ostatnie cztery cyfry numeru
     * @throws IllegalArgumentException jeli ktrakolwiek z liczb jest mniejsza
     * od zera, jeli piersze dwa elementy numeru maj wicej ni 3 cyfry, bd 
     * jeli ostatni element ma wicej ni cztery cyfry.
     */
    public PhoneNumber(int areaCode, int exchange, int extension) {
        setAreaCode((short)areaCode);
        setExchange((short)exchange);
        setExtension((short)extension);
    }
    
    /**
     * Pobranie pierszego elementu, czyli pierwszych trzech cyfr numeru.
     * @return pierwsze trzy cyfry numeru telefonicznego
     */
    public short getAreaCode() {
        return areaCode;
    }
    
    /**
     * Okrela pierwsze trzy cyfry numeru telefonicznego.
     * @param areaCode pierwsze trzy cyfry numeru telefonicznego
     * @throws IllegalArgumentException jeli podana liczba jest mniejsza od zera
     * lub jeli ma wicej ni trzy cyfry.
     */
    public void setAreaCode(short areaCode) {
        if (areaCode < 0 || areaCode > 999) {
            throw new IllegalArgumentException("Kod obszaru: " + areaCode);
        }
        this.areaCode = areaCode;
    }
    
    /**
     * Pobiera rodkowe trzy cyfry numeru.
     * @return rodkowe trzy cyfry numeru telefonicznego
     */
    public short getExchange() {
        return exchange;
    }
    
    /**
     * Okrela rodkowe trzy cyfry numeru telefonicznego..
     * @param exchange rodkowe trzy cyfry numeru telefonicznego
     * @throws IllegalArgumentException jeli podana liczba jest mniejsza od 
     * zera lub jeli ma wicej ni trzy cyfry.
     */
    public void setExchange(short exchange) {
        if (exchange < 0 || exchange > 999) {
            throw new IllegalArgumentException("zmiana: " + exchange);
        }
        this.exchange = exchange;
    }
    
    /**
     * Pobiera ostatnie cztery cyfry numeru telefonicznego.
     * @return ostatnie cztery cyfry numeru telefonicznego
     */
    public short getExtension() {
        return extension;
    }
    
    /**
     * Okrela cztery ostatnie cyfry numeru telefonicznego.
     * @param extension ostatnie cztery cyfry numeru telefonicznego.
     * @throws IllegalArgumentException jeli przekazana liczba jest mniejsza
     * od zera lub jeli ma wicej ni cztery cyfry.
     */
    public void setExtension(short extension) {
        if (extension < 0 || extension > 9999) {
            throw new IllegalArgumentException("wewntrzny: " + extension);
        }
        this.extension = extension;
    }
    
    public boolean equals(Object o) {
        if (o == this) {
            return true;
        }
        if (!(o instanceof PhoneNumber)) {
            return false;
        }
        PhoneNumber phoneNumber = (PhoneNumber)o;
        return phoneNumber.extension == this.extension &&
            phoneNumber.exchange == this.exchange &&
            phoneNumber.areaCode == this.areaCode;
    }
    
    public int hashCode() {
        int result = 17;    
        result = 37 * result + areaCode;
        result = 37 * result + exchange;
        result = 37 * result + extension;
        return result;
    }
    
    /**
     * Metoda zwraca acuch znakw reprezentujcy dany numer telefoniczny.
     * Numer skada si z czternastu znakw zapisany w nastpujcym 
     * formacie "(XXX) YYY-ZZZZ", gdzie XXX jest kodem obszaru (pierwsze trzy 
     * cyfry), YYY reprezentuje rodkowe trzy cyfry, a ZZZZ ostatnie cztery 
     * cyfry.  (Kada z duych liter oznacza jedn cyfr.)
     *
     * Jeli ktrakolwiek z czci numeru jest mniejsza ni dugo 
     * odpowiedniego pola, to zostanie uzupeniona o pocztkowe zera.
     * Na przyka jeli ostatnia cz numeru ma warto 123, to 
     * ostatnie cztery cyfry numeru bd mie posta: "0123".
     *
     * Naley zwrci uwag na znak odstpu, oddzielajcy nawias od 
     * rodkowej czci numeru.
     */
    public String toString() {
        return "(" + toPaddedString(areaCode, 3) + ") " +
            toPaddedString(exchange, 3) + "-" +
            toPaddedString(extension, 4);
    }
    
    /**
     * Metoda konwertuje numer telefoniczny do postaci acucha znakw
     * o okrelonej minimalnej dugoci i uzupenionego o pocztkowe zera.
     * Wykorzystywana jest metoda toString.
     * @param number numer telefoniczny zamieniany na acuch znakw
     * @param length minimalna dugo konwertowanego acucha
     * @return numer w postaci acucha znakw o okrelonej minimalnej dugoci. 
     * Jeli numer jest krtszy, to zostanie uzupeniony o zera dodane na jego 
     * pocztku.
     */
    private static String toPaddedString(int number, int length) {
        String s = Integer.toString(number);
        return ZEROS[length - s.length()] + s;
    }
    
    /**
     * Pomocnicza tablica acuchw skadajcych si z samych zer.
     * Uywana w metodzie toPaddedString.
     */
    private static String[] ZEROS = 
    {"", "0", "00", "000", "0000", "00000",
         "000000", "0000000", "00000000", "000000000"};
}
